require('./check-versions')()

const config = require(`${process.cwd()}/config`)
if (!process.env.NODE_ENV) {
  process.env.NODE_ENV = JSON.parse(config.dev.env.NODE_ENV)
}

const fs = require('fs')
const watch = require('watch')
const path = require('path')
const express = require('express')
const webpack = require('webpack')
const merge = require('webpack-merge')
const proxyMiddleware = require('http-proxy-middleware')
const portfinder = require('portfinder')
const webpackConfig = require('./webpack.dev.conf')

// 端口配置
const port = process.env.PORT || config.dev.port

// 代理地址配置
const proxyTable = config.dev.proxyTable

// 合并webpack配置项
const clientWebpack = config.dev.webpack
const webpackMerge = merge(clientWebpack, webpackConfig)

const app = express()
const compiler = webpack(webpackMerge)

/**
 * 监听src/pages文件变化
 */
function writeJSON (appJSON) {
  fs.writeFile(
    `${process.cwd()}/src/app.json`,
    JSON.stringify(appJSON, null, 2),
    'utf-8',
    function () {
      console.log('app.json已更新')
    })
}

if (config.dev.autoFiles) {
  watch.createMonitor(`${process.cwd()}/src/pages`, function (monitor) {
    monitor.files = monitor.files[`${process.cwd()}/src/pages`]
    monitor.on('created', function (f, stat) {
      if (stat.isDirectory()) {
        const dirPath = f.split('/')
        const dirName = dirPath[dirPath.length - 1]
        const files = ['index.vue', 'main.js', 'main.json']
        // 生成文件
        files.forEach(file => {
          fs.readFile(`${__dirname}/templates/${file}`, function (err, data) {
            if (err) throw err
            fs.writeFile(`${f}/${file}`, data, 'utf-8', function () {
              console.log(`${dirName}:${file}生成`)
            })
          })
        })
        // 添加app.json路径
        const appJSON = require(`${process.cwd()}/src/app.json`)
        if (!appJSON.pages) {
          appJSON.pages = []
        }
        appJSON.pages.push(`pages/${dirName}/main`)
        writeJSON(appJSON)
      }
    })

    monitor.on('removed', function (f) {
      const dirPath = f.split('/')
      const dirName = dirPath[dirPath.length - 1]
      const folderName = dirPath[dirPath.length - 2]

      // 添加app.json路径
      const appJSON = require(`${process.cwd()}/src/app.json`)
      let index = -1
      if (dirName) {
        const [name] = dirName.split('.')
        index = appJSON.pages.indexOf(`pages/${folderName}/${name}`)
      }

      if (index > -1) {
        appJSON.pages.splice(index, 1)
        writeJSON(appJSON)
      }
    })
  })
}

// proxy api requests
Object.keys(proxyTable).forEach(function (context) {
  let options = proxyTable[context]
  if (typeof options === 'string') {
    options = { target: options }
  }
  app.use(proxyMiddleware(options.filter || context, options))
})

app.use(require('connect-history-api-fallback')())

const staticPath = path.posix.join(config.dev.assetsPublicPath, config.dev.assetsSubDirectory)
app.use(staticPath, express.static('./static'))

// const uri = 'http://localhost:' + port

let _resolve = null
const readyPromise = new Promise(resolve => {
  _resolve = resolve
})

module.exports = new Promise(resolve => {
  portfinder.basePort = port
  portfinder.getPortPromise()
    .then(newPort => {
      if (port !== newPort) {
        console.log(`${port}端口被占用，开启新端口${newPort}`)
      }
      const server = app.listen(newPort, 'localhost')
      // for 小程序的文件保存机制
      require('webpack-dev-middleware-hard-disk')(compiler, {
        publicPath: webpackConfig.output.publicPath,
        quiet: true
      })
      resolve({
        ready: readyPromise,
        close: () => {
          server.close()
        }
      })
    }).catch(error => {
      console.log('没有找到空闲端口，请打开任务管理器杀死进程端口再试', error)
    })
})
